@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Cargill Incorporated
+// Copyright 2018-2022 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,17 +18,19 @@ use futures::{Future, IntoFuture};
 #[cfg(feature = "authorization")]
 use crate::admin::rest_api::CIRCUIT_WRITE_PERMISSION;
 use crate::admin::service::{AdminCommands, AdminServiceError};
-use crate::protocol;
 use crate::protos::admin::CircuitManagementPayload;
-use crate::rest_api::actix_web_1::{into_protobuf, Method, ProtocolVersionRangeGuard, Resource};
+use crate::rest_api::{
+    actix_web_1::{into_protobuf, Method, ProtocolVersionRangeGuard, Resource},
+    SPLINTER_PROTOCOL_VERSION,
+};
 use crate::service::ServiceError;
 
+const ADMIN_SUBMIT_PROTOCOL_MIN: u32 = 1;
+
 pub fn make_submit_route<A: AdminCommands + Clone + 'static>(admin_commands: A) -> Resource {
-    let resource =
-        Resource::build("/admin/submit").add_request_guard(ProtocolVersionRangeGuard::new(
-            protocol::ADMIN_SUBMIT_PROTOCOL_MIN,
-            protocol::ADMIN_PROTOCOL_VERSION,
-        ));
+    let resource = Resource::build("/admin/submit").add_request_guard(
+        ProtocolVersionRangeGuard::new(ADMIN_SUBMIT_PROTOCOL_MIN, SPLINTER_PROTOCOL_VERSION),
+    );
 
     #[cfg(feature = "authorization")]
     {
